@@ -9,15 +9,17 @@ from pathlib import Path
 from tempfile import NamedTemporaryFile, gettempdir
 from typing import Any, Dict, List, Union
 
+from gftools.builder.fontc import FontcArgs
 import networkx as nx
 import strictyaml
 import yaml
 from fontmake.font_project import FontProject
 from ninja import _program
 from ninja.ninja_syntax import Writer, escape_path
+from typing import Union
 
 from gftools.builder.file import File
-from gftools.builder.operations import OperationBase, known_operations
+from gftools.builder.operations import OperationBase, OperationRegistry
 from gftools.builder.operations.copy import Copy
 from gftools.builder.recipeproviders import get_provider
 from gftools.builder.schema import BASE_SCHEMA
@@ -36,7 +38,11 @@ class GFBuilder:
     config: dict
     recipe: Recipe
 
-    def __init__(self, config: Union[dict, str]):
+    def __init__(
+        self,
+        config: Union[dict, str],
+        fontc_args=FontcArgs(None),
+    ):
         if isinstance(config, str):
             parentpath = Path(config).resolve().parent
             with open(config, "r") as file:
@@ -54,8 +60,11 @@ class GFBuilder:
         else:
             self._orig_config = yaml.dump(config)
             self.config = config
+        fontc_args.modify_config(self.config)
 
-        self.writer = Writer(open("build.ninja", "w"))
+        self.known_operations = OperationRegistry(use_fontc=fontc_args.use_fontc)
+        self.ninja_file_name = fontc_args.build_file_name()
+        self.writer = Writer(open(self.ninja_file_name, "w"))
         self.named_files = {}
         self.used_operations = set([])
         self.graph = nx.DiGraph()
@@ -156,9 +165,9 @@ class GFBuilder:
 
     def operation_step_to_object(self, step):
         operation = step.get("operation") or step.get("postprocess")
-        if operation not in known_operations:
+        cls = self.known_operations.get(operation)
+        if cls is None:
             raise ValueError(f"Unknown operation {operation}")
-        cls = known_operations[operation]
         if operation not in self.used_operations:
             self.used_operations.add(operation)
             cls.write_rules(self.writer)
@@ -328,7 +337,9 @@ class GFBuilder:
     def draw_graph(self):
         import pydot
 
-        dot = subprocess.run(["ninja", "-t", "graph"], capture_output=True)
+        dot = subprocess.run(
+            ["ninja", "-t", "graph", "-f", self.ninja_file_name], capture_output=True
+        )
         graphs = pydot.graph_from_dot_data(dot.stdout.decode("utf-8"))
         targets = self.recipe.keys()
         if graphs and graphs[0]:
@@ -349,12 +360,29 @@ class GFBuilder:
             print("Could not parse ninja build file")
 
     def clean(self):
-        for file in ["./build.ninja", "./.ninja_log"]:
-            if os.path.exists(file):
-                os.remove(file)
+        if hasattr(self, "config") and isinstance(self.config, dict):
+            cleanUp = self.config.get("cleanUp")
+            if cleanUp == True:
+                print("Cleaning up temporary files...")
 
-        if os.path.exists("instance_ufos"):
-            shutil.rmtree("instance_ufos")
+                for file in [self.ninja_file_name, "./.ninja_log"]:
+                    if os.path.exists(file):
+                        os.remove(file)
+
+                # in the fontc/crater case it's possible that multiple instances of
+                # gftools builder are running at once, in the same directory,
+                # in which case we don't want to delete this out from under another process.
+                # since we should have deleted our own .ninja file above, if any still
+                # exist we will assume another instance of gftools is running.
+                if not any(path.endswith("ninja") for path in os.listdir()):
+                    if os.path.exists("instance_ufos"):
+                        shutil.rmtree("instance_ufos")
+                else:
+                    print("another .ninja file exists, leaving instance_ufos in place")
+
+                print("Done cleaning up temporary files")
+        else:
+            print("Configuration not found or invalid, skipping cleanup.")
 
 
 def main(args=None):
@@ -372,8 +400,27 @@ def main(args=None):
         help="Just generate and output recipe from recipe builder",
         action="store_true",
     )
+    parser.add_argument(
+        "--experimental-fontc",
+        help=f"Use fontc instead of fontmake. Argument is path to the fontc executable",
+        type=Path,
+    )
+
+    parser.add_argument(
+        "--experimental-simple-output",
+        help="generate a reduced set of targets, and copy them to the provided directory",
+        type=Path,
+    )
+
+    parser.add_argument(
+        "--experimental-single-source",
+        help="only compile the single named source file",
+        type=str,
+    )
+
     parser.add_argument("config", help="Path to config file or source file", nargs="+")
     args = parser.parse_args(args)
+    fontc_args = FontcArgs(args)
     yaml_files = []
     source_files = []
     for config in args.config:
@@ -395,7 +442,7 @@ def main(args=None):
             raise ValueError("Only one config file can be given for now")
         config = args.config[0]
 
-    pd = GFBuilder(config)
+    pd = GFBuilder(config, fontc_args=fontc_args)
     if args.generate:
         config = pd.config
         config["recipe"] = pd.recipe
@@ -408,4 +455,4 @@ def main(args=None):
         pd.draw_graph()
     if not args.no_ninja:
         atexit.register(pd.clean)
-        raise SystemExit(_program("ninja", []))
+        raise SystemExit(_program("ninja", ["-f", pd.ninja_file_name]))
